#-----------------------------------------------------------------------------
#
#   PyWeek 10 - About Screen
#
#-----------------------------------------------------------------------------

from albow.controls import Button
from albow.layout import Row
from albow.resource import get_image
from humerus.main_commands import MainCommandScreen
from game import game

story_page = get_image("about_story.png")
controls_page = get_image("about_controls.png")

pages = [story_page, controls_page]

page_bg_color = (0xff, 0xff, 0xff)

class AboutScreen(MainCommandScreen):

	margin = 10

	def __init__(self, *args, **kwds):
		MainCommandScreen.__init__(self, *args, **kwds)
		self.current_page_no = 0
		prev_button = Button("<- Back", enable = self.prev_exists, action = self.go_to_prev)
		next_button = Button("Next ->", enable = self.next_exists, action = self.go_to_next)
		play_button = Button("Play", action = self.go_to_play)
		buttons = Row([prev_button, play_button, next_button],
			height = self.height - pages[0].get_height(),
			bg_color = page_bg_color,
			spacing = 30)
		buttons.centerx = self.centerx
		buttons.bottom = self.bottom - self.margin
		self.add(buttons)
		
	def prev_exists(self):
		return self.current_page_no > 0
	
	def next_exists(self):
		return self.current_page_no < len(pages) - 1
	
	def go_to_prev(self):
		self.current_page_no -= 1
	
	def go_to_next(self):
		self.current_page_no += 1
	
	def go_to_play(self):
		if not game.play_in_progress():
			game.new_game()
		self.shell.show_play_screen()

	def draw(self, surf):
		surf.blit(pages[self.current_page_no], (0, 0))
