#-----------------------------------------------------------------------------
#
#   PyWeek 10 - Textures
#
#-----------------------------------------------------------------------------

from OpenGL import GL
import pygame
from albow.resource import get_image

def pow2up(a):
	b = 1
	while b < a:
		b <<= 1
	return b

#-----------------------------------------------------------------------------

class Texture(object):
	#  size        (width, height)   Size in pixels
	#  gl_tex_num  int               GL texture number
	
	tex_coords = [(0.0, 0.0), (1.0, 0.0), (1.0, 1.0), (0.0, 1.0)]

	def __init__(self, *args, **kwds):
		image = get_image(*args, **kwds)
		w, h = self.size = image.get_size()
		data = pygame.image.tostring(image, 'RGBA', True)
		self.gl_tex_num = GL.glGenTextures(1)
		self.bind()
		GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGBA, w, h, 0, GL.GL_RGBA,
			GL.GL_UNSIGNED_BYTE, data)
		GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_CLAMP)
		GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_CLAMP)
		GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR)
		GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR)

	def bind(self):
		#print "Texture: binding", self.gl_tex_num ###
		GL.glBindTexture(GL.GL_TEXTURE_2D, self.gl_tex_num)

	def quad_vertices(self, vertices, tex_coords = None, flip = 0):
#		if tex_coords:
#			print "Texture.quad_vertices:", vertices ###
#			print "...tex_coords =", tex_coords ###
		if not tex_coords:
			tex_coords = self.tex_coords
		for i in xrange(4):
			GL.glTexCoord2fv(tex_coords[i ^ flip])
			GL.glVertex2fv(vertices[i])
#		GL.glTexCoord2f(0.0, 0.0)
#		GL.glVertex2fv(vertices[0])
#		GL.glTexCoord2f(1.0, 0.0)
#		GL.glVertex2fv(vertices[1])
#		GL.glTexCoord2f(1.0, 1.0)
#		GL.glVertex2fv(vertices[2])
#		GL.glTexCoord2f(0.0, 1.0)
#		GL.glVertex2fv(vertices[3])

	def draw_quad(self, vertices, tex_coords = None, flip = 0):
		self.bind()
		GL.glBegin(GL.GL_QUADS)
		self.quad_vertices(vertices, tex_coords, flip)
		GL.glEnd()

	def draw_quad_with_foot_at(self, point, scale = 1.0, tex_coords = None,
			flip = 0):
		x, y = point
		w, h = self.size
		w *= scale
		h *= scale
		u = 0.5 * w
		vertices = [(x - u, y), (x + u, y), (x + u, y + h), (x - u, y + h)]
		self.draw_quad(vertices, tex_coords, flip)

	def draw_rect(self, rect, flip = 0):
		vertices = self.vertices_for_rect(rect)
		self.draw_quad(vertices, None, flip)
	
	def vertices_for_rect(self, rect):
		l, b, r, t = rect
		return [(l, b), (r, b), (r, t), (l, t)]

#-----------------------------------------------------------------------------

class TextureArray(Texture):

	def __init__(self, num_items, *args, **kwds):
		Texture.__init__(self, *args, **kwds)
		self.num_items = num_items
		w, h = self.size
		self.size = w / num_items, h
	
	def tex_coords_for_index(self, index):
		du = 1.0 / self.num_items
		u0 = index * du
		u1 = u0 + du
		return [(u0, 0.0), (u1, 0.0), (u1, 1.0), (u0, 1.0)]
	
	def draw_quad_with_foot_at(self, index, point, flip):
		tcoords = self.tex_coords_for_index(index)
		Texture.draw_quad_with_foot_at(self, point, tex_coords = tcoords, flip = flip)

	def draw_rect(self, index, rect, flip = 0):
		tcoords = self.tex_coords_for_index(index)
		vertices = self.vertices_for_rect(rect)
		self.draw_quad(vertices, tcoords, flip)
